package geniusweb.protocol.session.mopac.phase;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import geniusweb.actions.Action;
import geniusweb.actions.EndNegotiation;
import geniusweb.actions.Offer;
import geniusweb.actions.PartyId;
import geniusweb.inform.Inform;
import geniusweb.inform.YourTurn;
import geniusweb.issuevalue.Bid;
import geniusweb.protocol.ProtocolException;
import geniusweb.protocol.session.mopac.PartyStates;
import geniusweb.voting.VotingEvaluator;

public class OfferPhase extends DefaultPhase {

	public OfferPhase(Phase prevPhase, PartyStates partyStates, Long deadlinems,
			VotingEvaluator evaluator) {
		super(prevPhase, partyStates, deadlinems, evaluator);
	}

	@Override
	public OfferPhase with(PartyId actor, Action action, long timems) {
		try {
			checkAction(actor, action, timems);
		} catch (ProtocolException ex) {
			return this.with(ex);
		}
		return new OfferPhase(prevPhase, partyStates.with(action), deadline,
				evaluator);
	}

	@Override
	public Inform getInform() {
		return new YourTurn();
	}

	@Override
	public OfferPhase with(ProtocolException e) {
		System.out.println("Party kicked because of protocol exception:" + e);
		return new OfferPhase(prevPhase, partyStates.with(e), deadline,
				evaluator);
	}

	@Override
	public Phase finish() {
		return new OfferPhase(prevPhase, partyStates.finish(), deadline,
				evaluator);
	}

	@Override
	public VotingPhase checkedNext(long deadln) {
		return new VotingPhase(this, partyStates.flush(), deadln, evaluator);
	}

	public List<Bid> getOffers() {
		return partyStates.getActions(Offer.class).stream()
				.map(offer -> offer.getBid()).collect(Collectors.toList());
	}

	@Override
	public List<Class<? extends Action>> getAllowedActions() {
		return Arrays.asList(Offer.class, EndNegotiation.class);
	}

}
