package geniusweb.protocol.session.mopac.phase;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import geniusweb.actions.Action;
import geniusweb.actions.EndNegotiation;
import geniusweb.actions.PartyId;
import geniusweb.actions.Votes;
import geniusweb.inform.Agreements;
import geniusweb.inform.Inform;
import geniusweb.inform.OptIn;
import geniusweb.protocol.ProtocolException;
import geniusweb.protocol.session.mopac.PartyStates;
import geniusweb.voting.CollectedVotes;
import geniusweb.voting.VotingEvaluator;

public class OptInPhase extends DefaultPhase {

	protected OptInPhase(Phase prevPhase, PartyStates partyStates,
			Long deadlinems, VotingEvaluator evaluator) {
		super(prevPhase, partyStates, deadlinems, evaluator);
	}

	@Override
	public Inform getInform() {
		return new OptIn(((VotingPhase) prevPhase).getVotes());
	}

	@Override
	public Phase with(PartyId actor, Action action, long now) {
		try {
			checkAction(actor, action, now);
			if (action instanceof Votes)
				checkExtends(action);

			return new OptInPhase(prevPhase, partyStates.with(action), deadline,
					evaluator);

		} catch (ProtocolException e) {
			return this.with(e);
		}
	}

	/**
	 * Check that this action extends previous action.
	 * 
	 * @param action
	 * @param phase
	 * @throws ProtocolException if this action does not correctly extend
	 *                           previous vote.
	 */
	private void checkExtends(Action action) throws ProtocolException {

		Votes votes = (Votes) action;
		PartyId actor = action.getActor();
		// this actor is active so he must have voted in previous round.
		Votes prevVotes = ((VotingPhase) prevPhase).getVotes().stream()
				.filter(v -> v.getActor().equals(actor)).findFirst().get();
		if (!(votes.isExtending(prevVotes)))
			throw new ProtocolException("New votes " + action
					+ " does not extend previous vote " + prevVotes, actor);
	}

	@Override
	public OptInPhase with(ProtocolException e) {
		return new OptInPhase(prevPhase, partyStates.with(e), deadline,
				evaluator);
	}

	@Override
	public OptInPhase finish() {
		PartyStates states = partyStates.finish();
		Map<PartyId, Votes> votesmap = states.getActions(Votes.class).stream()
				.collect(Collectors.toMap(votes -> votes.getActor(), v -> v));
		CollectedVotes allvotes = new CollectedVotes(votesmap,
				states.getPowers());
		Agreements newAgree = evaluator.create(allvotes).getAgreements();
		if (!newAgree.getMap().isEmpty()) {
			System.out.println("detected new agreements");
		}
		PartyStates finalStates = states.with(newAgree);

		return new OptInPhase(prevPhase, finalStates, deadline, evaluator);
	}

	@Override
	public OfferPhase checkedNext(long newdeadline) {
		return new OfferPhase(this, partyStates.flush(), newdeadline,
				evaluator);
	}

	@Override
	public List<Class<? extends Action>> getAllowedActions() {
		return Arrays.asList(Votes.class, EndNegotiation.class);
	}

}
