package geniusweb.protocol.session.mopac.phase;

import java.util.Arrays;
import java.util.List;

import geniusweb.actions.Action;
import geniusweb.actions.EndNegotiation;
import geniusweb.actions.PartyId;
import geniusweb.actions.Votes;
import geniusweb.inform.Inform;
import geniusweb.inform.Voting;
import geniusweb.protocol.ProtocolException;
import geniusweb.protocol.session.mopac.PartyStates;
import geniusweb.voting.VotingEvaluator;

public class VotingPhase extends DefaultPhase {

	VotingPhase(Phase prevPhase, PartyStates partyStates, Long deadlinems,
			VotingEvaluator evaluator) {
		super(prevPhase, partyStates, deadlinems, evaluator);
	}

	@Override
	public VotingPhase with(PartyId actor, Action action, long now) {
		try {
			checkAction(actor, action, now);
		} catch (ProtocolException ex) {
			return this.with(ex);
		}
		return new VotingPhase(prevPhase, partyStates.with(action), deadline,
				evaluator);
	}

	@Override
	public Inform getInform() {
		return new Voting(((OfferPhase) prevPhase).getOffers(),
				partyStates.getPowers());
	}

	@Override
	public VotingPhase with(ProtocolException e) {
		return new VotingPhase(prevPhase, partyStates.with(e), deadline,
				evaluator);
	}

	@Override
	public VotingPhase finish() {
		return new VotingPhase(prevPhase, partyStates.finish(), deadline,
				evaluator);
	}

	@Override
	protected Phase checkedNext(long deadln) {
		return new OptInPhase(this, partyStates.flush(), deadln, evaluator);
	}

	@Override
	public List<Class<? extends Action>> getAllowedActions() {
		return Arrays.asList(Votes.class, EndNegotiation.class);
	}

}
